@@ -196,6 +196,7 @@ class BillQueryResult:
         self.code = None
         self.msg = None
         self.detail_list = []
+        self.total_size = None
 
     @classmethod
     def from_response_dict(cls, response_dict: dict) -> Optional['BillQueryResult']:
@@ -413,12 +414,14 @@ class AlipayCertClient:
         url = self.gateway + query
         return url
 
-    def create_bill_query_url(self, start_time: datetime, end_time: datetime):
+    def create_bill_query_url(self, start_time: datetime, end_time: datetime, page_no=1, page_size=2000):
         start_time_str = start_time.strftime('%Y-%m-%d %H:%M:%S')
         end_time_str = end_time.strftime('%Y-%m-%d %H:%M:%S')
         biz_content = {
             'start_time': start_time_str,
             'end_time': end_time_str,
+            'page_no': str(page_no),
+            'page_size': str(page_size),
         }
 
         params = self.public_params
