@@ -9,9 +9,48 @@ from django.db.models.signals import post_save, pre_save
 from django.utils import timezone
 from model_utils import FieldTracker
 
+from alipay.alipay import AlipayCertClient
 from alipay.signals import payment_succeed
 
 
+class Provider(models.Model):
+    ALIPAY_OPENAPI_GATEWAY = 'https://openapi.alipay.com/gateway.do?'
+    SIGN_TYPE_MD5 = "MD5"
+    SIGN_TYPE_RSA = "RSA"
+    SIGN_TYPE_RSA2 = "RSA2"
+    SIGN_TYPE_CHOICES = (
+        (SIGN_TYPE_MD5, SIGN_TYPE_MD5),
+        (SIGN_TYPE_RSA, SIGN_TYPE_RSA),
+        (SIGN_TYPE_RSA2, SIGN_TYPE_RSA2),
+    )
+
+    app_id = models.CharField(max_length=32)
+    private_key = models.TextField()
+    public_key = models.TextField(null=True, blank=True)  # 旧md5没有
+
+    app_cert = models.TextField(null=True, blank=True)  # content
+    alipay_root_cert = models.TextField(null=True, blank=True)
+
+    sign_type = models.CharField(max_length=12, choices=SIGN_TYPE_CHOICES, default=SIGN_TYPE_RSA)
+
+    seller_id = models.CharField(max_length=28)
+    seller_email = models.EmailField(max_length=100, null=True, blank=True)  # 迁移旧订单
+
+    gateway = models.CharField(max_length=200, default=ALIPAY_OPENAPI_GATEWAY)
+
+    def __str__(self):
+        return f'{self.app_id}#{self.seller_id}#{self.sign_type}|{self.seller_email}'
+
+    def __repr__(self):
+        return str(self)
+
+    def get_client(self):
+        return AlipayCertClient(self.app_id,
+                                self.private_key, self.public_key,
+                                self.app_cert, self.alipay_root_cert,
+                                self.gateway)
+
+
 class AlipayPayment(models.Model):
     TRADE_INIT = 'INIT'
     TRADE_WAIT_BUYER_PAY = 'WAIT_BUYER_PAY'
@@ -33,12 +72,14 @@ class AlipayPayment(models.Model):
 
     reference_id = models.CharField(max_length=256, null=True, blank=True)  # 业务相关id, 用来对应业务
 
+    provider = models.ForeignKey(Provider, on_delete=models.CASCADE)
+
     subject = models.CharField(max_length=256)
     body = models.TextField()  # max_length=1000
 
     amount_total = models.DecimalField(max_digits=10, decimal_places=2)
+    refund_total = models.DecimalField(max_digits=10, decimal_places=2, null=True, blank=True)
 
-    seller_email = models.EmailField(max_length=100)
     buyer_email = models.CharField(max_length=100, blank=True, null=True)
     buyer_id = models.CharField(max_length=16, blank=True, null=True)
 
@@ -55,6 +96,9 @@ class AlipayPayment(models.Model):
 
     tracker = FieldTracker(fields=['status'])
 
+    def __str__(self):
+        return f"{self.id}#{self.amount_total}|{self.status}"
+
     def is_succeed(self):
         return self.is_succeed_status(self.status)
 
@@ -119,3 +163,25 @@ class AlipayPayment(models.Model):
 
 post_save.connect(AlipayPayment.on_post_save, AlipayPayment)
 pre_save.connect(AlipayPayment.on_pre_save, AlipayPayment)
+
+
+class Refund(models.Model):
+    payment = models.ForeignKey(AlipayPayment, on_delete=models.CASCADE)
+    amount = models.DecimalField(max_digits=10, decimal_places=2)
+
+    out_trade_no = models.CharField(max_length=64, unique=True)  # 商户退款单号请求时为out_request_no返回时为out_trade_no
+
+    is_success = models.BooleanField(default=False)  # 返回时更新
+
+    note = models.TextField(null=True, blank=True)  # 会额外增加api失败结果
+
+    def __str__(self):
+        return f'{self.id}#P{self.payment.id}#{self.amount}|{self.is_success}'
+
+    def __repr__(self):
+        return str(self)
+
+    @staticmethod
+    def generate_refund_out_trade_no():
+        dt = timezone.make_naive(timezone.now())
+        return dt.strftime('%Y-%m-%d %H:%M:%S')
